@@ -33,13 +33,10 @@ class Preprocessor(object):
   '''
   def __init__(self,
                proteome,
-               split,
                preprocess_format,
                preprocessed_files_path='.',
-               gene_priority_proteome='',
-               versioned_ids = True):
-
-    assert split >= 2, 'k-sized split is invalid. Cannot be less than 2.'
+               versioned_ids = True,
+               gene_priority_proteome=''):
 
     if not preprocess_format in ('sql', 'pickle'):
       raise AssertionError('Unexpected value of preprocessing format:', preprocess_format)
@@ -49,7 +46,6 @@ class Preprocessor(object):
 
     self.proteome = proteome
     self.proteome_name = proteome.split('/')[-1].split('.')[0]
-    self.split = split
     self.preprocess_format = preprocess_format
     self.preprocessed_files_path = preprocessed_files_path
     self.gene_priority_proteome = gene_priority_proteome
@@ -65,14 +61,14 @@ class Preprocessor(object):
       kmers.append(seq[i:i+k])
     return kmers
 
-  def pickle_proteome(self, kmer_dict, names_dict):
+  def pickle_proteome(self, kmer_dict, names_dict, k):
     '''
     Takes the preprocessed proteome (below) and creates a pickle file for
     both k-mer and names dictionaries created. This is for compression and
     for being able to load the data in when a query is called.
     '''
     with open(os.path.join(self.preprocessed_files_path, self.proteome_name + '_' +
-              str(self.split) + 'mers.pickle'), 'wb') as f:
+              str(k) + 'mers.pickle'), 'wb') as f:
 
       pickle.dump(kmer_dict, f)
 
@@ -81,13 +77,13 @@ class Preprocessor(object):
 
       pickle.dump(names_dict, f)
 
-  def sql_proteome(self, kmer_dict, names_dict):
+  def sql_proteome(self, kmer_dict, names_dict, k):
     '''
     Takes the preprocessed proteome (below) and creates SQLite tables for both the
     k-mer and names dictionaries created. These SQLite tables can then be used
     for searching. This is much faster for exact matching.
     '''
-    kmers_table = self.proteome_name + '_' + str(self.split) + 'mers'
+    kmers_table = self.proteome_name + '_' + str(k) + 'mers'
     names_table = self.proteome_name + '_names'
 
     conn = sqlite3.connect(os.path.join(self.preprocessed_files_path, self.proteome_name + '.db'))
@@ -114,7 +110,7 @@ class Preprocessor(object):
     c.close()
     conn.close()
 
-  def preprocess(self):
+  def preprocess(self, k):
     '''
     Method which preprocessed the given proteome, by splitting each protein into k-mers
     and assigninga unique index to each unique k-mer within each protein. This is done by
@@ -123,11 +119,14 @@ class Preprocessor(object):
     guarantees a unique index for each and every possible k-mer. Also, each protein #
     assigned is also mappedto the protein ID to be read back later after searching.
     '''
+    assert k >= 2, 'k-sized split is invalid. Cannot be less than 2.'
+
     proteome = parse_fasta(self.proteome)
     kmer_dict = {}
     names_dict = {}
     protein_count = 1
 
+    # get all gene priority IDs to check for them
     if self.gene_priority_proteome:
       gene_priority_proteome_ids = []
       gene_priority_proteome = parse_fasta(self.gene_priority_proteome)
@@ -136,18 +135,13 @@ class Preprocessor(object):
         gene_priority_proteome_ids.append(protein_id)
 
     for protein in proteome:
-      kmers = self.split_protein(str(protein.seq), self.split)
+      kmers = self.split_protein(str(protein.seq), k)
       for i in range(len(kmers)):
         if kmers[i] in kmer_dict.keys():
           kmer_dict[kmers[i]].append(protein_count * 100000 + i) # add index to k-mer list
         else:
           kmer_dict[kmers[i]] = [protein_count * 100000 + i]     # create entry for new k-mer
 
-      if self.gene_priority_proteome:
-        gene_priority = 1 if protein_id in gene_priority_proteome_ids else 0
-      else:
-        gene_priority = None
-
       # grab UniProt ID which is usually in the middle of two vetical bars
       try:
         protein_id = protein.id.split('|')[1]
@@ -180,6 +174,11 @@ class Preprocessor(object):
       except AttributeError:
         pe_level = None
 
+      if self.gene_priority_proteome:
+        gene_priority = 1 if protein_id in gene_priority_proteome_ids else 0
+      else:
+        gene_priority = None
+
       if self.versioned_ids:
         try:
           versioned_id = re.search('SV=(.*)', protein.description).group(1)
@@ -192,11 +191,11 @@ class Preprocessor(object):
 
     # store data based on format specified
     if self.preprocess_format == 'pickle':
-      self.pickle_proteome(kmer_dict, names_dict)
+      self.pickle_proteome(kmer_dict, names_dict, k)
     elif self.preprocess_format == 'sql':
-      self.sql_proteome(kmer_dict, names_dict)
+      self.sql_proteome(kmer_dict, names_dict, k)
 
-    return kmer_dict, names_dict
+    return 0
 
 
 # run via command line
@@ -208,8 +207,8 @@ def parse_arguments():
   parser.add_argument('-k', '--kmer_size', type=int, required=True)
   parser.add_argument('-f', '--format', required=True)
   parser.add_argument('-P', '--preprocessed_files_path', default='.')
-  parser.add_argument('-g', '--gene_priority_proteome', default='')
   parser.add_argument('-v', '--versioned_ids', type=bool, default=True)
+  parser.add_argument('-g', '--gene_priority_proteome', default='')
 
   args = parser.parse_args()
 
@@ -218,5 +217,5 @@ def parse_arguments():
 def run():
   args = parse_arguments()
 
-  Preprocessor(args.proteome, args.kmer_size, args.format, args.preprocessed_files_path,
-             args.gene_priority_proteome, args.versioned_ids).preprocess()
\ No newline at end of file
+  Preprocessor(args.proteome, args.format, args.preprocessed_files_path,
+               args.versioned_ids, args.gene_priority_proteome).preprocess(args.kmer_size)
